# -*- coding: utf-8 -*-
"""
View generating a PDF document with the badges.
"""
import os
from tempfile import mkstemp
import itertools

from reportlab.lib.units import cm
from reportlab.lib.utils import ImageReader
from reportlab.pdfgen.canvas import Canvas
from math import floor

from cubicweb import ValidationError
from cubicweb.view import EntityView
from cubicweb.web import formfields, Redirect
from cubicweb.web.form import FormViewMixIn
from cubicweb.predicates import is_instance, one_line_rset, match_user_groups
from cubicweb.web.controller import Controller
from cubicweb.web.action import Action
from cubicweb.web.formwidgets import SubmitButton

_ = unicode

# XXX to put in the form
P_WIDTH = 29.7*cm
P_HEIGHT = 21.0*cm
P_MARGIN = 1*cm
B_WIDTH = 9.0*cm
B_HEIGHT = 5.5*cm
LOGO_MAX_WIDTH = B_WIDTH
LOGO_MAX_HEIGHT = 1.5*cm
B_H_MARGIN = 0.1*cm
B_V_MARGIN = 0.5*cm

class BadgePageCanvas(object):
    "Generate and save a pdf page of badges"

    def __init__(self, filename, logo_filename, author='', title=''):
        self.h_badges_num = int(floor((P_WIDTH - 2 * P_MARGIN) / B_WIDTH))
        self.v_badges_num = int(floor((P_HEIGHT - 2 * P_MARGIN) / B_HEIGHT))
        self.badges_per_page = self.h_badges_num * self.v_badges_num
        if self.badges_per_page == 0:
            raise ValueError("Badge size too big")
        self.h_margin = (P_WIDTH - (self.h_badges_num * B_WIDTH)) / 2.0
        self.v_margin = (P_HEIGHT - (self.v_badges_num * B_HEIGHT)) / 2.0
        self.line_width = B_WIDTH - 2 * B_H_MARGIN
        # Logo specs
        self.logo = None
        self.img_width = None
        self.img_height = None
        self.__set_logo(logo_filename)
        self.create_canvas(author, title, filename)

    def __set_logo(self, logo_filename=None):
        "Load the logo and set variables"
        if logo_filename is not None:
            self.logo = ImageReader(logo_filename)
            self.img_width, self.img_height = self.logo.getSize()
            if self.img_width > LOGO_MAX_WIDTH:
                self.img_width *= LOGO_MAX_WIDTH / self.img_width
            if self.img_height > LOGO_MAX_HEIGHT:
                self.img_height = LOGO_MAX_HEIGHT
        else:
            self.logo = None

    def create_canvas(self, author, title, filename):
        "Canvas initialization"
        self.canvas = Canvas(filename)
        self.canvas.setPageSize((P_WIDTH, P_HEIGHT))
        self.canvas.title = title
        self.canvas.author = author
        self.canvas.filename = filename

    def draw_org(self, organization, lxc, lyc):
        "Print the current organization"
        font_size = 16
        while font_size > 0:
            self.canvas.setFont("Helvetica", font_size)
            if self.canvas.stringWidth(organization) <= self.line_width:
                break
            else:
                font_size -= 1
        self.canvas.drawCentredString(lxc + B_WIDTH / 2.0,
                                      lyc + B_V_MARGIN,
                                      organization)

    def draw_name(self, person, lxc, lyc):
        "Print name and surname (one line if short, two lines if long)"
        long_name = person.dc_long_title()
        font_size = 20
        top = B_HEIGHT - self.img_height
        bottom = B_V_MARGIN + font_size
        self.canvas.setFont("Helvetica-Bold", font_size)
        if person.is_chair_at:
            self.canvas.setFillColorRGB(0, .37, 0.01)
        elif person.leads:
            self.canvas.setFillColorRGB(0, 0, .53)
        else:
            self.canvas.setFillColorRGB(0, 0, 0)
        if self.canvas.stringWidth(long_name) <= self.line_width:
            height = lyc + ((top - bottom)/2.0 + bottom) - font_size / 1.5
            self.canvas.drawCentredString(lxc + B_WIDTH / 2.0,
                                          height,
                                          long_name)
        else:
            while font_size > 0:
                self.canvas.setFont("Helvetica-Bold", font_size)
                if (self.canvas.stringWidth(person.firstname) <= self.line_width and
                    self.canvas.stringWidth(person.surname) <= self.line_width):
                    break
                else:
                    font_size -= 1
            height = ((top - bottom)  / 2.0 + bottom) - font_size * 1.15
            self.canvas.drawCentredString(lxc + B_WIDTH / 2.0,
                                          lyc + height + font_size * 1.3,
                                          person.firstname)
            self.canvas.drawCentredString(lxc + B_WIDTH / 2.0,
                                          lyc + height,
                                          person.surname)
        self.canvas.setFillColorRGB(0, 0, 0)

    def fill_page(self, persons):
        "Fill the page with badges"
        self.canvas.setLineWidth(0.25)
        tuples = [[i, j, None] for i in range(self.h_badges_num) \
                  for j in range(self.v_badges_num)]
        tuples[-1][-1] = True
        index_iterator = itertools.cycle(tuples)
        self.canvas.setLineWidth(0.25)
        for person in persons:
            x_index, y_index, page = index_iterator.next()
            lxc = self.h_margin + B_WIDTH * x_index
            lyc = self.v_margin + B_HEIGHT * y_index
            self.canvas.grid([lxc, lxc + B_WIDTH],
                             [lyc, lyc + B_HEIGHT])
            if self.logo:
                self.canvas.drawImage(self.logo,
                                      lxc,
                                      lyc - self.img_height + B_HEIGHT,
                                      LOGO_MAX_WIDTH,
                                      LOGO_MAX_HEIGHT,
                                      preserveAspectRatio=True,
                                      anchor = 'sw')
            # Draw company and name
            if person.representing:
                self.draw_org(person.representing, lxc, lyc)
            self.draw_name(person, lxc, lyc)
            # If needed, generate a new page.
            if page:
                self.canvas.showPage()
                self.canvas.setLineWidth(0.25)
        self.canvas.save()
        return self.canvas


class GenerateBadgeAction(Action):
    __regid__ = 'generate_badge_action'
    title = _('generate badges')
    __select__ = is_instance('Conference') & one_line_rset() & match_user_groups('managers')

    def url(self):
        return self.cw_rset.get_entity(0, 0).absolute_url(vid='badgesform')

class BadgesForm(FormViewMixIn, EntityView):
    """ ask for optional parameters to generate a badge"""
    __regid__ = 'badgesform'
    __select__ = is_instance('Conference') & one_line_rset() & match_user_groups('managers')

    def entity_call(self, conference):
        self.w('<h2>%s</h2>' % self._cw._('Generate badges'))
        self.w('<div>%s</div>' % self._cw._('You can select a new logo, '
                                            'else the site\'s logo will be taken'))
        form = self._cw.vreg['forms'].select('base', self._cw, rset=self.cw_rset,
                                             form_renderer_id='base',
                                             entity=conference,
                                             domid='badgesform',
                                             action=self._cw.build_url('dobadges'),
                                             __errorurl=conference.absolute_url(vid='badgesform'),
                                             form_buttons=[SubmitButton()])
        form.append_field(formfields.FileField(name='logo_field',
                                               label=self._cw.__('logo file')))
        renderer = form.default_renderer()
        self.w(form.render(renderer=renderer))

class BadgesController(Controller):
    __regid__ = 'dobadges'

    def publish(self, rset=None):
        try:
            _filename, stream = self._cw.form['logo_field']
            fd, logo = mkstemp()
            logo_file = open(logo, 'w')
            logo_file.write(stream.read())
            logo_file.close()
            os.close(fd)
            is_logo_tempfile = True
        except (KeyError, ValueError):
            logo = self._cw.uiprops['LOGO']
            is_logo_tempfile = False
        conference = self._cw.execute('Any C WHERE C eid %(c)s',
                                      {'c': self._cw.form['eid']}).get_entity(0, 0)
        raise Redirect(conference.absolute_url(vid='badgeview', logo=logo,
                                               is_logo_tempfile=is_logo_tempfile))

class EmptyPerson(object):
    """ fake class to fake people with empty names to do handmade badges """
    representing = u' '
    is_chair_at = False
    leads = False
    firstname = u''
    surname = u''

    def dc_long_title(self):
        return u' '

class BadgeView(EntityView):
    __regid__ = 'badgeview'
    title = _('generate badges')
    __select__ = is_instance('Conference') & one_line_rset() & match_user_groups('managers')
    content_type = "application/pdf"
    templatable = False
    binary = True
    filename = 'badges.pdf'

    def set_request_content_type(self):
        """overriden to set a .pdf filename"""
        self._cw.set_content_type(self.content_type,
                                  filename=self.filename)

    def call(self):
        conference = self.cw_rset.get_entity(0, 0)
        logo = self._cw.form.get('logo')
        if not logo:
            return
        is_logo_tempfile = self._cw.form.get('is_logo_tempfile') == 'True'
        page = BadgePageCanvas(self.filename, logo,
                               self._cw.user.dc_long_title(), 'Badges',
                               )
        page.fill_page(self.get_users(conference))
        page.fill_page([EmptyPerson(),]*9) #empty badges for last minute attendees
        if is_logo_tempfile:
            os.remove(logo)
        self.w(open(self.filename).read())

    def get_users(self, conference):
        """
        return users which are register to the conference
        """
        rql = ('Any U ORDERBY N WHERE U is CWUser, U surname N?, EXISTS(S is ShoppingCart, '
               'S buyer U, S in_state ST, ST name IN ("awaiting payment", "checked out"), '
               'S items_in_cart I, I item_type IT, C has_shoppingitemtype IT, C eid %(x)s)')
        return self._cw.execute(rql, dict(x=conference.eid)).entities()



# copyright 2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-cmt views/forms/actions/components for web ui"""

import datetime
from logilab.mtconverter import xml_escape

from cubicweb.view import EntityView
from cubicweb.predicates import (is_instance, match_view, match_context,
                                 authenticated_user)
from cubicweb.web import Redirect, component
from cubicweb.web.component import Layout
from cubicweb.web.views import primary, uicfg

from cubes.conference.views.startup import ConferenceIndexView
from cubes.conference.views.forms import subject_reg_open_conf_vocabulary
from cubes.shoppingcart.views import ShoppingCartPrimaryView
from cubes.shoppingcart.views import ShoppingCartEntityFormRenderer

uicfg.autoform_field_kwargs.tag_subject_of(('ShoppingCart', 'book_conf', 'Conference'),
                                           {'choices': subject_reg_open_conf_vocabulary})
uicfg.actionbox_appearsin_addmenu.tag_subject_of(('Conference', 'has_shoppingitemtype', '*'), True)
uicfg.actionbox_appearsin_addmenu.tag_subject_of(('ShoppingCart', 'billing', '*'), True)
uicfg.primaryview_section.tag_subject_of(('ShoppingCart', 'billing', '*'), 'hidden')
uicfg.primaryview_section.tag_object_of(('ShoppingCart', 'billing', 'BillingAddress'), 'hidden')
uicfg.autoform_section.tag_subject_of(('ShoppingCart', 'billing', '*'), 'main', 'inlined')

_ = unicode

# this message is needed to have it appear in translation files
_('creating ShoppingCart (ShoppingCart buyer CWUser %(linkto)s)')

REPLACE_LIST = []


class CmtIndexView(ConferenceIndexView):

    def call(self):
        super(CmtIndexView, self).call()
        rset = self._cw.execute('Any X ORDERBY CD DESC LIMIT 1 WHERE X is BlogEntry, X creation_date CD')
        if rset:
            for post in rset.entities():
                self.w(post.view('primary'))

REPLACE_LIST.append((CmtIndexView, ConferenceIndexView))


class ConferenceRegistrationComponent(component.CtxComponent):
    __regid__ = 'confregistrationbox'
    context = 'left'
    title = _(u'Conference Registration')
    order = 0
    label = None

    def init_rendering(self):
        reg_open = self._cw.execute('Any C WHERE C is Conference, C reg_open True')
        if not reg_open:
            raise component.EmptyComponent()

        # BUGGY: what if there is more than one conf ?
        self.conf = self._cw.entity_from_eid(reg_open[0][0])

        register_now       = _('Register now for the conference !')
        pay_to_confirm     = _('Pay now to confirm your registration !')
        you_are_registered = _('You are registered for the conference')

        if self._cw.session.anonymous_session:
            if reg_open:
                self.label = register_now
        else:
            has_cart = self._cw.execute('Any SC WHERE U eid %(x)s, SC buyer U', {'x': self._cw.user.eid})
            items = self._cw.execute('Any I WHERE U eid %(u)s, SC buyer U, SC items_in_cart I, '
                                     'I item_type T, C has_shoppingitemtype T, C eid %(c)s',
                                     {'u': self._cw.user.eid, 'c': self.conf.eid})
            if has_cart:
                if items:
                    state = has_cart.get_entity(0, 0).cw_adapt_to('IWorkflowable').state
                    if state == 'checked out':
                        self.label = you_are_registered
                    else:
                        self.label = pay_to_confirm
                else:
                    self.label = register_now
            elif reg_open:
                 self.label = register_now
        if self.label is None:
            raise component.EmptyComponent()

    def render_body(self, w):
        label = self._cw._(self.label)
        w(u'<div class="conferenceBox">')
        w(u'<div id="confRegister">')
        w(u'<a href="%s">%s</a>' % (xml_escape(self.conf.absolute_url(vid='conf-registration')), xml_escape(label)))
        w(u'</div></div>')
        # debugging purpose
        return self.label


class ConferenceRegistrationLayout(Layout):
    __select__ = Layout.__select__ & match_context('left') & match_view('confregistrationbox')

    # raw rendering instead of default box layout (no extra div)
    def render(self, w):
        if self.init_rendering():
            view = self.cw_extra_kwargs['view']
            view.render_body(w)


class CmtShoppingCartEntityFormRenderer(ShoppingCartEntityFormRenderer):

    explanation_text = _(u'To register for the conference, add to your shopping '
                         'cart the admission fees listed below.')

REPLACE_LIST.append((CmtShoppingCartEntityFormRenderer, ShoppingCartEntityFormRenderer))

from cubes.cmcicpay import cmcic
from cubes.cmcicpay.views import get_tpe

class CmtShoppingCartPrimaryView(ShoppingCartPrimaryView):

    def reg_state(self, entity):
        if entity.items_in_cart:
            if entity.in_state[0].name == 'in progress':
                return _('not payed for')
            return _('confirmed')
        else:
            return _('empty')

    def render_entity_title(self, entity):
        reg = self.reg_state(entity)
        conf = entity.book_conf[0]
        title = self._cw._('Your registration to %s is %s') % (conf.dc_title(), self._cw._(reg))
        self.w(u'<h1>%s (#%s)</h1>' % (xml_escape(title), entity.eid))

    def render_entity_relations(self, entity):
        super(CmtShoppingCartPrimaryView, self).render_entity_relations(entity)
        reg = self.reg_state(entity)
        btn_modify = (u'<div><a href="%s">%s</a></div>' % (
                self._cw.build_url(entity.eid, vid='edition'),
                self._cw._('modify your registration')))
        if entity.billing:
            self.w(u'<p>%s</p>' % self._cw._('Billing address'))
            entity.billing[0].view('primary', w=self.w)
        else:
            add_url = self._cw.build_url('add/BillingAddress',
                                         __linkto='billing:%d:object' % entity.eid,
                                         __redirectpath='shoppingcart/%d' % entity.eid,
                                         __redirectvid='primary')
            self.w(u'<p><a href="%s">%s</a></p>' % (xml_escape(add_url), self._cw._('Add a billing address')))
        if reg == 'not payed for':
            rset = self._cw.execute('Any C WHERE C has_shoppingitemtype T, I item_type T, '
                                    'SC items_in_cart I, SC eid %(x)s', dict(x=entity.eid))
            conf = rset.get_entity(0,0)
            total = sum(item.quantity*item.item.price for item in entity.items_in_cart)
            tpe = get_tpe(self._cw)
            payreq = cmcic.PaymentRequest()
            payreq.reference = str(entity.eid)
            payreq.amount = "%.2f" % total
            payreq.currency = "EUR"
            payreq.description = conf.title
            payreq.date = datetime.datetime.now().strftime("%d/%m/%Y:%H:%M:%S")
            payreq.lang = "EN"
            #payreq.email = entity.buyer[0].primary_email
            payreq.email = "test@test.zz"
            payreq.url_root = self._cw.base_url()
            payreq.url_ok = entity.absolute_url(__message='The payment succeeded.')
            payreq.url_err = entity.absolute_url(__message='The payment failed.')
            label = self._cw._('pay now %s %s to confirm your registration') % (
                payreq.amount, payreq.currency)
            icon = u'<img alt="OK_ICON" src="%s"/>' % self._cw.uiprops['OK_ICON']
            submit = u'<button type="submit" class="validateButton">%s %s</button>' % (icon, label)
            self.w(u'<table width="100%%"><tr><td>%s</td><td>%s</td></tr></table>' % (
                    cmcic.html_form(tpe, payreq, submit),
                    btn_modify))
        elif reg == 'empty':
            self.w(u'<br /><p style="background-color: #F5E5EE;">')
            self.w(self._cw._('You have not added admission fees for the conference or the tutorials to this shopping cart.<br />'
                              'You are <b>NOT REGISTERED</b> at this point.'))
            self.w(u'</p>')
            self.w(btn_modify)
        else:
            self.w(u'<p><a href="%s">%s</a></p>' % (entity.book_conf[0].absolute_url(), self._cw._('Go to the conference schedule')))

REPLACE_LIST.append((CmtShoppingCartPrimaryView, ShoppingCartPrimaryView))

# XXX move this upstream into cw
def set_field_order(etype, attrs):
    for index, attr in enumerate(attrs):
        uicfg.autoform_field_kwargs.tag_subject_of(
            (etype, attr, '*'), {'order': index})
        uicfg.primaryview_display_ctrl.tag_subject_of(
            (etype, attr, '*'), {'order': index})

set_field_order('BillingAddress',
                ('organisation', 'street', 'street2', 'postalcode', 'city',
                 'country', 'your_ref',))

class BillingAddressPrimaryView(primary.PrimaryView):
    __select__ = primary.PrimaryView.__select__ & is_instance('BillingAddress')

    def render_entity_title(self, entity):
        pass

# registration view
class ConfRegistrationView(EntityView):
    __regid__ = 'conf-registration'
    __select__ = EntityView.__select__ & is_instance('Conference')

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        self.w(u'<h1>Registration for conference %s</h1>' % entity.dc_title())
        self.w(u'<div>%s</div>' % self._cw._(
                'You need to create an account and sign in '
                'before you can book the conference'))
        view = self._cw.vreg['views'].select('login_or_register', self._cw)
        self.w(view.render(title=False))

class ConfRegistrationAuthView(EntityView):
    __regid__ = 'conf-registration'
    __select__ = EntityView.__select__ & is_instance('Conference') & authenticated_user()

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        rset = self._cw.execute('Any S WHERE S buyer U, S book_conf C, U eid %(u)s, C eid %(c)s',
                                {'u': self._cw.user.eid, 'c': entity.eid})
        if rset:
            entity = rset.get_entity(0,0)
            url = entity.absolute_url()
        else:
            linkto = 'buyer:%s:subject' % self._cw.user.eid
            url = self._cw.build_url('add/ShoppingCart', __linkto=linkto)
        raise Redirect(url)


## urls

from cubicweb.web.views.urlrewrite import SimpleReqRewriter, rgx

class ConfRewrite(SimpleReqRewriter):
    rules = [
        (rgx('/conference/([^/]+)/registration'),
         dict(rql='Any C WHERE C is Conference, C url_id "%(id)s"' % {'id': r'\1'},
              vid='conf-registration')),
         ]

def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, [new for new,old in REPLACE_LIST])
    for new, old in REPLACE_LIST:
        vreg.register_and_replace(new, old)



# copyright 2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# CubicWeb is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# CubicWeb is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with CubicWeb.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement
from cubicweb.utils import UStringIO
from cubicweb.devtools.testlib import CubicWebTC
from cubicweb.web.component import EmptyComponent


class ConfRegistrationComponentTC(CubicWebTC):
    regid = 'confregistrationbox'

    def create_conf(self, cnx, reg_open=True):
        pa = cnx.create_entity('PostalAddress',
                               street=u"ma rue",
                               postalcode=u"42",
                               city=u'ma ville')
        conf = cnx.create_entity('Conference', title=u"my conf",
                                 reg_open=reg_open, take_place_at=pa)
        cnx.commit()
        return conf

    def setUp(self):
        super(ConfRegistrationComponentTC, self).setUp()

    def test_boolean_field(self):
        with self.admin_access.repo_cnx() as cnx:
            self.assertEqual(len(cnx.execute('Any C WHERE C is Conference')), 0)
            pa = cnx.create_entity('PostalAddress',
                                   street=u"ma rue",
                                   postalcode=u"42",
                                   city=u'ma ville')
            conf = cnx.create_entity('Conference', title=u"my conf",
                                     reg_open=True, take_place_at=pa)
            cnx.commit()
            self.assertEqual(len(cnx.execute('Any C WHERE C is Conference, C reg_open True')), 1)

    def test_create_conf(self):
        with self.admin_access.repo_cnx() as cnx:
            self.assertEqual(len(cnx.execute('Any C WHERE C is Conference')), 0)
            conf = self.create_conf(cnx)
            self.assertTrue(conf.reg_open)
            self.assertEqual(len(cnx.execute('Any C WHERE C is Conference, C reg_open True')), 1)
            self.assertEqual(conf.take_place_at[0].street, u"ma rue")
            conf = self.create_conf(cnx, reg_open=False)
            self.assertEqual(len(cnx.execute('Any C WHERE C is Conference, C reg_open False')), 1)
            self.assertFalse(conf.reg_open)
            self.assertEqual(len(cnx.execute('Any C WHERE C is Conference')), 2)

    def test_no_open_registration(self):
        with self.admin_access.web_request() as req:
            component = self.vreg['ctxcomponents'].select(self.regid, req)
            self.assertEqual(component.context, "left")
            self.assertRaises(EmptyComponent, component.init_rendering)
            self.create_conf(req.cnx, reg_open=False)
            self.assertRaises(EmptyComponent, component.init_rendering)

    def test_anonymous_session(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_conf(cnx)

        with self.new_access('anon').web_request() as req:
            self.assertTrue(req.session.anonymous_session)
            component = self.vreg['ctxcomponents'].select(self.regid, req)
            component.init_rendering()
            label = component.render_body(UStringIO)
        self.assertEqual(label, _('Register now for the conference !'))

    def test_items_not_found(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_conf(cnx)

        with self.admin_access.web_request() as req:
            component = self.vreg['ctxcomponents'].select(self.regid, req)
            component.init_rendering()
            label = component.render_body(UStringIO)
        self.assertEqual(label, _('Register now for the conference !'))

    def test_items_found_and_cart_is_checked_out(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_conf(cnx)

        you_are_registered = _('You are registered for the conference')
        self.skipTest(you_are_registered)

    def test_items_found_and_cart_not_checked_out(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_conf(cnx)

        pay_to_confirm = _('Pay now to confirm your registration !')
        self.skipTest(pay_to_confirm)


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



add_cube('cmcicpay')
add_cube('openidrelay')
add_relation_definition('ShoppingCart', 'book_conf', 'Conference', ask_confirm=True)
add_relation_definition('Conference', 'has_shoppingitemtype', 'ShoppingItemType', ask_confirm=True)
sync_schema_props_perms('quantity')



# copyright 2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-cmt schema"""


from yams.buildobjs import RelationDefinition, EntityType, String

from cubicweb.schema import ERQLExpression

from cubes.shoppingcart.schema import ShoppingItem
from cubes.conference.schema import user_owns_subject,  user_owns_object

ShoppingItem.get_relation('quantity').default = 1

class book_conf(RelationDefinition):
    subject = 'ShoppingCart'
    object = 'Conference'
    cardinality = '1*'

class has_shoppingitemtype(RelationDefinition):
    subject = 'Conference'
    object = 'ShoppingItemType'
    cardinality = '*1'

class BillingAddress(EntityType):
    __permissions__ = {
        'read':   ('managers', ERQLExpression('X owned_by U'),),
        'add':    ('managers', 'users',),
        'update': ('managers', 'owners',),
        'delete': ('managers', 'owners',),
        }
    organisation = String(required=True, maxsize=256)
    street = String(required=True, maxsize=256)
    street2 = String(maxsize=256)
    postalcode = String(required=True, maxsize=256)
    city = String(required=True, maxsize=256)
    country = String(required=True, maxsize=256)
    your_ref = String(maxsize=256)

class billing(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users'),
        'add':    ('managers', user_owns_subject()),
        'delete': ('managers', user_owns_subject()),
        }
    subject = 'ShoppingCart'
    object = 'BillingAddress'
    cardinality = '?1'
    composite = 'subject'



# pylint: disable-msg=W0622
"""cubicweb-cmt application packaging information"""

modname = 'cmt'
distname = 'cubicweb-cmt'

numversion = (0, 8, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LCL'
copyright = 'Copyright (c) 2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.'

author = 'LOGILAB S.A. (Paris, FRANCE)'
author_email = 'contact@logilab.fr'

description = 'conference management tool'

web = 'http://www.cubicweb.org/project/%s' % distname

pyversions = ['2.4']

__depends__ = {'cubicweb': '>= 3.19.0',
               'cubicweb-blog': None,
               'cubicweb-cmcicpay': None,
               'cubicweb-conference': '>= 0.12.0',
               'cubicweb-forgotpwd': '>= 0.2.0',
               'cubicweb-openidrelay': None,
               'cubicweb-registration': None,
               'cubicweb-shoppingcart': '>= 0.4.0',
               'cubicweb-tag': None,
               }

# packaging ###

from os import listdir as _listdir
from os.path import join, isdir, exists, dirname
from glob import glob

THIS_CUBE_DIR = join('share', 'cubicweb', 'cubes', modname)

def listdir(dirpath):
    return [join(dirpath, fname) for fname in _listdir(dirpath)
            if fname[0] != '.' and not fname.endswith('.pyc')
            and not fname.endswith('~')
            and not isdir(join(dirpath, fname))]

data_files = [
    # common files
    [THIS_CUBE_DIR, [fname for fname in glob('*.py') if fname != 'setup.py']],
    ]
# check for possible extended cube layout
for dirname in ('entities', 'views', 'sobjects', 'hooks', 'schema', 'data', 'i18n', 'migration'):
    if isdir(dirname):
        data_files.append([join(THIS_CUBE_DIR, dirname), listdir(dirname)])

